@@ -5,10 +5,6 @@
 import sys
 import os
 import subprocess
-
-os.environ['CFLAGS'] = '-I/usr/local/include'
-os.environ['LDFLAGS'] = '-L/usr/local/lib'
-
 try:
     cmd = './configure --prefix=`pwd`/dist '
     for arg in sys.argv[1:]:
@@ -25,4 +21,4 @@ try:
 except OSError, e:
     print >>sys.stderr, 'execution failed', e
 except KeyboardInterrupt:
-    pass
+    print
