@@ -4,42 +4,31 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## The classes for writing and reading cache
 
-import md5
-import time
-
-try:
-    import sqlite3
-    _sqlite3_found = 1
-except ImportError:
-    _sqlite3_found = 0
-
-try:
-    from clive.newt import snack as newt
-except ImportError, err:
-    raise SystemExit('error: %s' % err)
+__all__ = ['Cache','Browse']
 
 from clive.path import ConfigDir
 from clive.progress import Progress
 from clive.unicode import tostr
-
-__all__ = ['Cache','Browse']
+from clive.modules import Modules
+from clive.opts import Options
 
 ## The class that wraps Cache access
 class Cache:
@@ -47,24 +36,28 @@ class Cache:
     ## Constructor
     def __init__(self, say):
         self._say = say
-        if not _sqlite3_found: return
-        self._conn = sqlite3.connect(ConfigDir().cachefile())
-        self._conn.row_factory = sqlite3.Row
-        self._cur = self._conn.cursor()
-        self._upgrade_if_needed()
-        self._create_table()
+        self._md5 = Modules().getinst('md5')
+        self._time = Modules().getinst('time')
+        self._pysqlite_avail = \
+            Modules().check('pysqlite')['installed']
+
+        if self._pysqlite_avail:
+            self._pysqlite = Modules().getinst('pysqlite')
+            self._conn = self._pysqlite.connect(ConfigDir().cachefile())
+            self._conn.row_factory = self._pysqlite.Row
+            self._cur = self._conn.cursor()
+            self._upgrade_if_needed()
+            self._create_table()
 
     ## Destructor
     def __del__(self):
-        if not _sqlite3_found: return
-        self._cur.execute('vacuum')
-        self._cur.close()
+        if self._pysqlite_avail:
+            self._cur.execute('vacuum')
+            self._cur.close()
 
-    ## Write new cache entry
-    #
-    # Supresses any insertion errors quietly.
+    ## Write a new cache entry
     def write(self, v_info):
-        hash = md5.new(v_info['url'].strip('&fmt=18')).hexdigest()
+        hash = self._md5.new(v_info['url'].strip('&fmt=18')).hexdigest()
         r = self.read(v_info['url'].strip('&fmt=18'))
         if not r:
             try:
@@ -78,12 +71,12 @@ class Cache:
                     'cache_lowq, ' \
                     'cache_url)' \
                     'values(?,?,?,?,?,?,?,?)',
-                    (hash, time.time(), v_info['page_title'], 
+                    (hash, self._time.time(), v_info['page_title'], 
                     v_info['length_bytes'], v_info['xurl'], v_info['v_id'],
                     v_info['low_quality'], v_info['url'].strip('&fmt=18')))
-            except sqlite3.IntegrityError, err:
+            except self._pysqlite.IntegrityError, err:
                 pass # Ignore any "url hash not unique" errors
-            except sqlite3.OperationalError, err:
+            except self._pysqlite.OperationalError, err:
                 pass # Ignore any "database locked" etc. errors
         else: # Update existing record (e.g. if quality setting was changed)
             if r['cache_lowq'] != v_info['low_quality']:
@@ -95,7 +88,7 @@ class Cache:
                     'cache_vid=?, ' \
                     'cache_lowq=?, ' \
                     'cache_url=? where cache_hash=?',
-                    (time.time(), v_info['page_title'],
+                    (self._time.time(), v_info['page_title'],
                     v_info['length_bytes'],
                     v_info['xurl'], v_info['v_id'], v_info['low_quality'],
                     v_info['url'].strip('&fmt=18'), hash))
@@ -104,7 +97,7 @@ class Cache:
     ## Read cache entry for URL
     def read(self, url):
         try:
-            hash = md5.new(url.strip('&fmt=18')).hexdigest()
+            hash = self._md5.new(url.strip('&fmt=18')).hexdigest()
             self._cur.execute('select ' \
                 'cache_title,' \
                 'cache_length,' \
@@ -112,7 +105,7 @@ class Cache:
                 'cache_vid, ' \
                 'cache_lowq ' \
                 'from cache where cache_hash=?', (hash,))
-        except sqlite3.OperationalError, err:
+        except self._pysqlite.OperationalError, err:
             if 'no such column' in err.message:
                 # Should database upgrade fail for some reason, re-create
                 # the table. This will, of course, remove all existing
@@ -126,27 +119,25 @@ class Cache:
         return self._cur.fetchone()
 
     ## Check if extraction URL has expired
-    #
-    # Some hosts have their extraction URLs expire after awhile.
     def has_xurl_expired(self, url):
         hosts = [('youtube.', 90),('dailymotion.', 2)]
         for (host, expires_min) in hosts: # Only these hosts req. check
             if host in url.lower():
-                hash = md5.new(url.strip('&fmt=18')).hexdigest()
+                hash = self._md5.new(url.strip('&fmt=18')).hexdigest()
                 self._cur.execute('select ' \
                     'cache_time from cache where cache_hash=?', (hash,))
                 row = self._cur.fetchone()
                 if not row: return 0 # Not found -> not expired
-                elapsed = time.time() - float(row['cache_time'])
-                gmt = time.gmtime(elapsed)
+                elapsed = self._time.time() - float(row['cache_time'])
+                gmt = self._time.gmtime(elapsed)
                 return (gmt[3]*60 + gmt[4]) >= expires_min
         return 0 # No such host -> not expired
 
     ## Updates extraction URL for an URL (hash)
     def update_expired_xurl(self, url, xurl):
-        hash = md5.new(url.strip('&fmt=18')).hexdigest()
+        hash = self._md5.new(url.strip('&fmt=18')).hexdigest()
         self._cur.execute('update cache set cache_time=?, cache_xurl=? ' \
-            'where cache_hash=?', (time.time(), xurl, hash))
+            'where cache_hash=?', (self._time.time(), xurl, hash))
         self._conn.commit()
 
     ## Select all cache entries, return cursor
@@ -155,17 +146,20 @@ class Cache:
         return self._cur
 
     ## Clear cache data
-    def clear(self):
+    def clear(self, enable_confirm):
+        Modules().getinst('pysqlite') # SystemExits if needed
         try:
             self._cur.execute('select * from cache limit 1')
             row = self._cur.fetchone()
             if not row:
                 self._say('error: cache: nothing to clear')
             else:
-                a = raw_input('> clear cache, are you sure? (y/N): ')
+                a = 'y'
+                if enable_confirm:
+                    a = raw_input('> clear cache, are you sure? (y/N): ')
                 if len(a) and a.lower()[0] == 'y':
                     self._recreate_table()
-        except sqlite3.OperationalError, err:
+        except self._pysqlite.OperationalError, err:
             if err.message == 'no such table' != -1: pass
 
     def _recreate_table(self):
@@ -184,13 +178,11 @@ class Cache:
                 'cache_vid text not null,' \
                 'cache_lowq integer not null,' \
                 'cache_url text not null)')
-        except sqlite3.OperationalError, err:
+        except self._pysqlite.OperationalError, err:
             # Ignore only "table exists" errors only
             if 'table exists already' in err.message:
                 pass
 
-    ## Destructor
-    # Upgrades table if it appears to be outdated (e.g. missing columns)
     def _upgrade_if_needed(self):
         try:
             self._cur.execute('select * from cache limit 1')
@@ -202,11 +194,10 @@ class Cache:
             if len(row) < 9:
                 # Must be > 0.4.10 <= 0.4.16 database
                 self._alter_table("'cache_url' text not null default ''")
-        except sqlite3.OperationalError, err:
+        except self._pysqlite.OperationalError, err:
             # Ignore "no such table" errors only
             if err.message == 'no such table' != -1: pass
 
-    # Alter table by adding a new column
     def _alter_table(self, s):
         self._cur.execute('alter table cache add column %s' % s)
         self._conn.commit()
@@ -215,17 +206,17 @@ class Cache:
 class Browse:
 
     ## Constructor
-    def __init__(self, opts, say):
+    def __init__(self, say):
         self._last = None # Selection before hitting "Resize"
-        self._opts = opts
+        self._opts = Options()._opts
         self._say = say
+        self._newt = Modules().getinst('newt')
 
     ## Browse URLs for re-extraction
     def browse(self):
         self._cache = Cache(self._say)
         if not self._cache.select_all().fetchone():
             raise SystemExit('error: no cache data')
-
         while 1:
             sel = self._main(self._cache.select_all())
             if sel: break
@@ -234,7 +225,7 @@ class Browse:
     def _main(self, cur):
         sel = []
         try:
-            scr = newt.SnackScreen()
+            scr = self._newt.SnackScreen()
             sel = self._show_cache(scr, cur)
         finally:
             scr.finish()
@@ -253,7 +244,7 @@ class Browse:
     def _show_cache(self, scr, cur):
         w = scr.width - (scr.width/6)
         ww = w-12
-        ct = newt.CheckboxTree(scr.height/2, scroll=1)
+        ct = self._newt.CheckboxTree(scr.height/2, scroll=1)
         i = 0
         while 1:
             row = cur.fetchone()
@@ -261,7 +252,7 @@ class Browse:
             if len(row['cache_url']): # Ignore any < 0.4.16 entries
                 t = row['cache_title'][:w]
                 try:
-                    t = tostr(t, self._opts)
+                    t = tostr(t)
                 except UnicodeDecodeError, err:
                     continue # Skip item if decoding fails for some reason
                 ct.append(t, row['cache_url'])
@@ -271,12 +262,12 @@ class Browse:
                     sel = True
                 u = row['cache_url'][:ww]
                 u += (' (%s)' % Progress()._byteshuman(row['cache_length']))
-                ct.addItem(u, (i, newt.snackArgs['append']),
+                ct.addItem(u, (i, self._newt.snackArgs['append']),
                     row['cache_url'], selected=sel)
                 i += 1                    
-        g = newt.GridForm(scr, 'Cache: Videos found', 1,2)
+        g = self._newt.GridForm(scr, 'Cache: Videos found', 1,2)
         g.add(ct, col=0, row=0, padding=(0,0,0,1))
-        b = newt.ButtonBar(scr,
+        b = self._newt.ButtonBar(scr,
             [('Resize',2),('Cancel',0), ('Extract',1)], compact=0)
         g.add(b, col=0, row=01, padding=(0,0,0,0))
         self._ct = ct
