@@ -4,38 +4,34 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## The classes for parsing runtime configuration file
 
-import os
-import sys
-import time
-import errno
-
-from clive.path import ConfigDir, Autodetect
-
 __all__ = ['ConfigParser']
 
+from clive.path import ConfigDir, Autodetect
+from clive.modules import Modules
 
 ## The class that wraps config file parsing
 class ConfigParser:
     _config = {}
-    _line_no = 0
-
+    _line = 0
+    _os = Modules().getinst('os')
     _lookup = {
         'enable_extract' : {
             'default':'YES',
@@ -112,7 +108,7 @@ class ConfigParser:
                 'e.g. "/usr/local/bin/xclip -o")',
         },
         'output_savedir' : {
-            'default':os.getcwd(),
+            'default':_os.getcwd(),
             'accepts':['__any__'],
             'opts_key':'output_savedir',
             'desc':'## Save directory for downloaded videos, use absolute ' \
@@ -160,29 +156,17 @@ class ConfigParser:
             'opts_key':'http_throttle',
             'desc':'## Uncomment to override the default 0 (unlimited, KB/s)',
         },
-        'youtube_user' : {
+        'youtube_login' : {
             'default':None,
             'accepts':['__any__'],
-            'opts_key':'youtube_username',
-            'desc':'## Youtube log-in username',
+            'opts_key':'youtube_login',
+            'desc':'## Youtube login info in format "username:password"',
         },
-        'youtube_password' : {
+        'dmotion_login' : {
             'default':None,
             'accepts':['__any__'],
-            'opts_key':'youtube_password',
-            'desc':'## Youtube log-in password',
-        },
-        'dmotion_user' : {
-            'default':None,
-            'accepts':['__any__'],
-            'opts_key':'dmotion_username',
-            'desc':'## Dailymotion log-in username',
-        },
-        'dmotion_password' : {
-            'default':None,
-            'accepts':['__any__'],
-            'opts_key':'dmotion_password',
-            'desc':'## Dailymotion log-in password',
+            'opts_key':'dmotion_login',
+            'desc':'## Dailymotion login info in format "username:password"',
         },
         'feed_url' : {
             'default':[],
@@ -204,54 +188,50 @@ class ConfigParser:
         try:
             f = open(ConfigDir().rcfile(), 'r')
         except IOError, e:
+            errno = Modules().getinst('errno')
             if e.errno == errno.ENOENT: return
             else: raise e
-
         self._config = {}
-        found_vars = []
-        self._line_no = 0
-        for ln in f.readlines():
-            self._line_no += 1
+        found_cmds = []
+        self._line = 0
+        # Parse file line by line
+        for ln in f:
             ln = ln.rstrip('\n')
-            if len(ln) == 0: continue
-            if ln.startswith('#'): continue
-            try:
-                (var,val) = ln.split('=', 1)
-                if not val.startswith('"') or not val.endswith('"'):
-                    self._err('expected quoted value (e.g. cmd="value")')
-                    continue
-                val = val.lstrip().rstrip()
-                val = val.strip('"')
-                try:
-                    d = self._lookup[var]
-                    if val not in d['accepts']:
-                        if len(val) == 0:
-                            if d['accepts'][0] != '__any_or_empty__':
-                                self._err('cannot be an empty value')
-                        elif d['accepts'][0] not \
-                            in ['__any__','__any_or_empty__']:
-                                self._err('invalid value (%s), expected %s' % \
-                                    (val, str(d['accepts'])))
-                except KeyError:
-                    self._warn('unrecognized command (%s)' % var)
-                    continue
-                if var != 'feed_url':
-                    self._config[var] = val
-                    found_vars.append(var)
-                else: # RSS/Atom feed URL
-                    try:
-                        self._config[var].append(val)
-                    except KeyError:
-                        self._config[var] = [val,]
-                    if var not in found_vars: # Append only once
-                        found_vars.append(var)
-            except ValueError:
-                self._err('expected \'command="value"\'')
+            self._line += 1
+            # Skip comments and empty lines
+            if len(ln) == 0 or ln.startswith('#'):
                 continue
-
-        self._set_defaults(found_vars)
+            try:                
+                (cmd,val) = ln.split('=',1)
+            except ValueError:
+                self._warn('invalid format: expected $(cmd)="$(value)"')
+                continue
+            if not val.startswith('"') and not val.endswith('"'):
+                self._warn('invalid format: unquoted value')
+                continue
+            val = val.strip().strip('"')
+            if cmd not in self._lookup:
+                self._warn('unrecognized command')
+                continue
+            d = self._lookup[cmd]
+            if val not in d['accepts']:
+                if len(val) == 0:
+                    if d['accepts'][0] != '__any_or_empty__':
+                        self._err('expected non-empty value')
+                    elif d['accepts'][0] not in ['__any__','__any_or_empty__']:
+                        self._err('expected %s' % str(d['accepts']))
+            if cmd == 'feed_url': # Exception for RSS/Atom feeds
+                try:
+                    self._config[cmd].append(val)
+                except KeyError:
+                    self._config[cmd] = [val,]
+                if cmd not in found_cmds:
+                    found_cmds.append(cmd)
+            else:
+                self._config[cmd] = val
+                found_cmds.append(cmd)
+        self._set_defaults(found_cmds)
         self._filter_yesno()
-
         return self._config
 
     ## Return default value for the requested option key
@@ -265,28 +245,29 @@ class ConfigParser:
     ## (Re)writes config file with default values
     #
     # Will prompt the user if the file exists already.
-    def rewrite_config(self, say):
-        if os.path.exists(ConfigDir().rcfile()):
-            if raw_input('> rewrite existing config file? (y/N): ').lower() \
-                == 'y':
-                os.remove(ConfigDir().rcfile())
+    def rewrite_config(self, say, enable_confirm):
+        if self._os.path.exists(ConfigDir().rcfile()):
+            a = 'y'
+            if enable_confirm:
+                a = raw_input('> rewrite existing config file? (y/N): ')
+            if len(a) > 0 and a.lower()[0] == 'y':
+                self._os.remove(ConfigDir().rcfile())
             else:
                 return
-
+        # Autodetect: player, ffmpeg, xclip
         self._lookup['path_player']['default'] = \
-            Autodetect().locate('vlc', '%i', say)
-
+            Autodetect().locate(['vlc','xine','mplayer'], '%i', say)
         self._lookup['path_ffmpeg']['default'] = \
             Autodetect().locate('ffmpeg', '-y -i %i %o', say)
-
         self._lookup['path_xclip']['default'] = \
             Autodetect().locate('xclip', '-o', say)
-
-        v = os.getenv('http_proxy')
+        # Autodetect: http_proxy env. setting
+        v = self._os.getenv('http_proxy')
         if v: say('found: %s (http_proxy)' % v)
         self._lookup['http_proxy']['default'] = v
-
+        # Write file
         f = open(ConfigDir().rcfile(), 'w')
+        time = Modules().getinst('time')
         f.write('## Created: %s\n\n' % time.asctime())
         f.write('## Lines that begin with "##" try to explain what is ' \
             'going on.\n## Lines that begin with just "#" are disabled ' \
@@ -302,11 +283,11 @@ class ConfigParser:
         f.close()
         say('done.')
             
-    def _set_defaults(self, found_vars=None):
+    def _set_defaults(self, found_cmds=None):
         for var in self._lookup:
             set = 0
-            if found_vars:
-                if var not in found_vars:
+            if found_cmds:
+                if var not in found_cmds:
                     set = 1
             else:
                 set = 1
@@ -322,11 +303,7 @@ class ConfigParser:
         self._err(warning, 'warn')
 
     def _err(self, error, _type='error'):
-        print >>sys.stderr, '%s:config:%d: %s' % (_type, self._line_no, error)
+        sys = Modules().getinst('sys')
+        fn = ConfigDir().rcfile()
+        print >>sys.stderr, '%s:%d: %s' % (fn, self._line, error)
         if _type == 'error': raise SystemExit
-
-if __name__ == '__main__':
-    conf = ConfigParser()
-    conf.load()
-    conf.generate_file()
-
