@@ -4,33 +4,25 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## The "nomad" core classes
 
-import os
-import gzip
-import subprocess
-import string
-import urllib2
-
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+__all__ = ['Nomad']
 
 import clive as _clive
 from clive.console import Console
@@ -44,83 +36,81 @@ from clive.recall import Recall
 from clive.login import Login
 from clive.rss import FeedParser
 from clive.scan import Scan
+from clive.util import transl_embed_url
+from clive.modules import Modules
 from clive.error import CliveError, CliveNoMediaError
-try:
-    from clive.urlgrabber.grabber import URLGrabber, URLGrabError
-except ImportError, err:
-    raise SystemExit('error: %s' % err)
-
-__all__ = ['Nomad']
-
 
 ## The "nomad" engine and core class of clive
 class Nomad:
 
     ## Entry point for the engine
-    # \param self The object pointer
-    # \param opts The parsed runtime options returned by optparse.OptionParser
-    # \param args The parsed command line arguments (optparse.OptionParser)
-    # \param say The pointer to a stdout method used throughout life-time
-    def run(self, opts, args, say):
-        self._opts = opts
-        self._args = args
+    def run(self, say):
         self._say = say
-
+        o = Options()
+        self._opts = o._opts
+        self._args = o._args
+        (self._URLGrabber, self._URLGrabError) = Modules().getinst('urlgrabber')
+        self._os = Modules().getinst('os')
+        self._subprocess = Modules().getinst('subprocess')
+        self._string = Modules().getinst('string')
         self._cache = Cache(say)
-
         self._found_urls = []
         self._error_urls = []
         self._dld_videos = []
         self._encoded_videos = []
-
         self._cookie_opener = None
         self._run_terminate = False # If true, terminate before stdin read
 
-        if opts.check_updates:
+        if self._opts.check_updates: # Check for updates
             self._check_updates()
 
-        if opts.write_conf:
+        if self._opts.write_conf: # Write conf
             self._write_conf()
 
-        if opts.clear_cache:
+        if self._opts.clear_cache: # Clear cache
             self._clear_cache()
 
-        if opts.clear_last:
-            self._run_terminate = 1
+        if self._opts.clear_last: # Clear recall
             self._clear_last()
 
-        if self._run_terminate:
+        if self._opts.list_hosts:
+            self._list_hosts()
+
+        if self._run_terminate: # Terminate if flagged
             raise SystemExit
-        if opts.enable_cache_browse:
+
+        if self._opts.enable_cache_browse: # Browse cache -- grab URLs
             # raw_urls: User selection from cache
             raw_urls = self._browse_cache()
-        else:
-            raw_urls = self._get_raw_urls(args)
+        else: # Otherwise -- grab URLs elsewhere
+            raw_urls = self._get_raw_urls(self._args)
 
-        if opts.enable_rss:
+        if self._opts.enable_rss: # We're parsing feeds
             # raw_urls: RSS URLs (input) -> Video URLs (user selection)
             raw_urls = self._parse_rss(raw_urls)
 
-        if opts.enable_scan:
+        if self._opts.enable_scan: # We're scanning URLs for video links
             self._scan(raw_urls)
-        else:
+        else: # Not scanning: Login if info is provided and check grabbed URLs
             self._login_if_needed(raw_urls)
             self._check_raw_urls(raw_urls)
 
-        if opts.enable_extract and not opts.emit_csv:
+        if self._opts.enable_extract and not self._opts.emit_csv:
+            # We're extracting
             self._dl()
-            if opts.reencode_format:
+            if self._opts.reencode_format:
                 self._re_encode()
-            if opts.play_format:
+            if self._opts.play_format:
                 self._play()
         else:
-            if not opts.enable_extract:
+            # Not extracting: only pretend or emit as csv
+            if not self._opts.enable_extract:
                 self._no_extract()
-            elif opts.emit_csv:
+            elif self._opts.emit_csv:
                 self._emit_csv()
 
     def _parse_rss(self, raw_urls):
-        p = FeedParser(self._opts, self._say, self._get_proxy())
+        p = FeedParser(self._say, self._get_proxy())
         return p.parse(raw_urls)
 
     def _reset_found_urls(self):
@@ -130,27 +120,25 @@ class Nomad:
         return self._found_urls
 
     def _browse_cache(self):
-        return Browse(self._opts, self._say).browse()
+        return Browse(self._say).browse()
 
     def _scan(self, raw_urls):
         callbacks = (self._check_url, self._reset_found_urls,
             self._get_found_urls, self._login_if_needed)
-        s = Scan(self._opts, self._say, self._get_proxy(),
+        s = Scan(self._say, self._get_proxy(),
             callbacks, self._cache)
         self._found_urls = s.scan(raw_urls)
         self._show_queue()
 
     def _login_if_needed(self, raw_urls):
+        urllib2 = Modules().getinst('urllib2')
         self._cookie_opener = \
             urllib2.build_opener(urllib2.HTTPCookieProcessor())
-
-        l = Login(self._opts, self._cookie_opener,
+        l = Login(self._cookie_opener,
             self._get_proxy, self._say)
-
         a = [('youtube.', l.youtube),
             ('metacafe.', l.metacafe),
             ('dailymotion.', l.dmotion)]
-
         found = 0
         for (host,method) in a:
             for url in raw_urls:
@@ -161,34 +149,38 @@ class Nomad:
 
     def _check_updates(self):
         self._run_terminate = 1
-        Update(self._say, self._opts, self._get_proxy()).check()
+        Update(self._say, self._get_proxy()).check()
 
     def _write_conf(self):
         self._run_terminate = 1
-        ConfigParser().rewrite_config(self._say)
+        ConfigParser().rewrite_config(self._say, self._opts.enable_confirm)
 
     def _clear_cache(self):
         self._run_terminate = 1
-        self._cache.clear()
+        self._cache.clear(self._opts.enable_confirm)
 
     def _clear_last(self):
         self._run_terminate = 1
-        Recall().clear(self._say)
+        Recall().clear(self._say, self._opts.enable_confirm)
+
+    def _list_hosts(self):
+        self._run_terminate = 1
+        for host in PageParser()._supported_hosts:
+            _host = host[0]
+            if _host.endswith('.'): _host += '*'
+            print _host
 
     def _no_extract(self):
         if len(self._found_urls) > 0:
             self._say('==> Passed:', show_always=1)
-
         for v_info in self._found_urls:
             text = v_info['url'][:40].ljust(40)
-            fn = os.path.basename(v_info['output_file'])
+            fn = self._os.path.basename(v_info['output_file'])
             text += fn[:27].rjust(28)
             text += v_info['length'].rjust(11)
             self._say(text, show_always=1)
-
         if len(self._error_urls) > 0:            
             self._say('==> Failed:', show_always=1)
-
         for video in self._error_urls:
             (url, err) = video
             text = url[:45].ljust(46)
@@ -198,15 +190,12 @@ class Nomad:
     def _emit_csv(self):
         if len(self._found_urls) > 0:
             self._say('==> Passed:', show_always=1)
-
         for v_info in self._found_urls:
             self._say('OK: "%s","%s","%s","%s"\n' % (v_info['url'],
-                v_info['xurl'], os.path.basename(v_info['output_file']),
+                v_info['xurl'], self._os.path.basename(v_info['output_file']),
                 v_info['length']), show_always=1)
-
         if len(self._error_urls) > 0:            
             self._say('==> Failed:', show_always=1)
-
         for video in self._error_urls:
             self._say('FAILED: "%s","%s"\n' % (video[0],video[1]),
                 show_always=1)
@@ -214,13 +203,12 @@ class Nomad:
     def _dl(self):
         if len(self._found_urls) == 0:
             raise SystemExit('error: nothing to extract.')
-
         for v_info in self._found_urls:
             try:
                 if not v_info['skip_extraction']:
                     self._dl_video(v_info)
                 self._dld_videos.append((v_info['xurl'], v_info['output_file']))
-            except URLGrabError, err:
+            except self._URLGrabError, err:
                 self._say('%s [%s]' % (err.strerror, v_info['url']), is_error=1)
                 self._error_urls.append((v_info['url'], err.strerror))
     
@@ -228,39 +216,34 @@ class Nomad:
         for (url, file) in self._dld_videos:
             output_file = '%s.%s' % (file, self._opts.reencode_format)
             output_file_stderr = '%s.stderr' % output_file
-
             self._say('re-encode: %s -> %s' % (
-                os.path.basename(file), os.path.basename(output_file)))
-
+                self._os.path.basename(file),
+                self._os.path.basename(output_file)))
             cmd = self._opts.ffmpeg.replace('%','$')
             d = {'i':'"%s"'%file, 'o':'"%s"'%output_file}
-            cmd = string.Template(cmd).substitute(d)
+            cmd = self._string.Template(cmd).substitute(d)
             cmd += ' >/dev/null'
             cmd += ' 2>"%s"' % output_file_stderr
-
-            rc = subprocess.call(cmd, shell=1)
+            rc = self._subprocess.call(cmd, shell=1)
             if rc != 0:
                 self._say('warn: ffmpeg returned an error, see %s' % ( \
                     output_file_stderr))
             else:
-                try: os.remove('%s' % output_file_stderr)
+                try: self._os.remove('%s' % output_file_stderr)
                 except: pass
                 self._encoded_videos.append((file, output_file))
 
     def _play(self):
         play_videos = self._dld_videos
-
         if self._opts.play_format != 'src':
             play_videos = self._encoded_videos
-
         for (ignore, file) in play_videos:
-            self._say('play: %s' % os.path.basename(file))
+            self._say('play: %s' % self._os.path.basename(file))
             cmd = self._opts.player.replace('%','$')
-            cmd = string.Template(cmd).substitute({'i':'"%s"'%file})
+            cmd = self._string.Template(cmd).substitute({'i':'"%s"'%file})
             cmd += ' >/dev/null'
             cmd += ' 2>&1'
-
-            rc = subprocess.call(cmd, shell=1)
+            rc = self._subprocess.call(cmd, shell=1)
             if rc != 0:
                 self._say('warn: player returned error %d' % rc)
 
@@ -268,7 +251,7 @@ class Nomad:
         if not args:
             if self._opts.enable_rss and not self._opts.enable_xclip_paste:
                 return self._opts.feed_url
-            return Console(self._opts, self._say).read()
+            return Console(self._say).read()
         else:
             return args
     
@@ -286,16 +269,11 @@ class Nomad:
             raise CliveError('error: unsupported host (%s)' % url)
 
     def _check_url(self, url, index):
-        if not url.startswith('http://'):
+        # Append "http://' if it's missing
+        if not url.lower().startswith('http://'):
             url = 'http://' + url
-
-        # ytube
-        url = url.replace('/v/','/watch?v=')
-
-        # vgoogle
-        url = url.replace('/googleplayer.swf?docId=',
-            '/videoplay?docid=')
-
+        # Convert embedded video URL to video page URL
+        url = transl_embed_url(url)
         try:
             if 'last.fm' in url.lower():
                 id_start = url.rfind('/') + 1
@@ -305,43 +283,38 @@ class Nomad:
                 else:
                     raise CliveError('error: only youtube format lastfm ' \
                         'videos are supported (%s); see clive(1) HOSTS' % url)
-
             if 'youtube.' in url.lower():
                 if not self._opts.enable_low_quality:
                     url += '&fmt=18'
-
             self._check_host(url)
-
             cache_row = None
             xurl_expired = False
-
             if self._opts.enable_cache:
                 cache_row = self._cache.read(url)
                 if cache_row:
                     xurl_expired = self._cache.has_xurl_expired(url)
                     if xurl_expired:
                         cache_row = None
-
             title = data = xurl = v_id  = ''
             file_length = -1
-
             low_quality = self._opts.enable_low_quality
             if cache_row: # Compare cache entry with current option
                 if cache_row['cache_lowq'] != low_quality:
                     cache_row = None # Different, disable cache use
-
             if not cache_row:
                 # Fetch video page
-                g = URLGrabber(
+                g = self._URLGrabber(
                     user_agent = self._opts.http_agent,
                     http_headers = (('accept-encoding','gzip'),),
-                    progress_obj = ProgressVerify(self._opts, index),
+                    progress_obj = ProgressVerify(index),
                     throttle = self._opts.http_throttle,
                     proxies = self._get_proxy(),
                     opener = self._cookie_opener)
                 o = g.urlopen(url)
                 data = o.read()
                 if o.hdr.get('content-encoding') == 'gzip':
+                    gzip = Modules().getinst('gzip')
+                    StringIO = Modules().getinst('StringIO')
                     data = gzip.GzipFile(fileobj=StringIO(data)).read()
             else:
                 # Use cache values
@@ -350,36 +323,30 @@ class Nomad:
                 xurl = cache_row['cache_xurl']
                 v_id = cache_row['cache_vid']
                 low_quality = cache_row['cache_lowq']
-                ProgressVerify(self._opts, index).from_cache(url, cache_row)
-
+                ProgressVerify(index).from_cache(url, cache_row)
             url_data = {
-                'url':url,
-                'data':data,
-                'title':title,
+                'page_url':url,
+                'page_data':data,
+                'page_title':title,
+                'page_xurl':xurl,
+                'video_id':v_id,
+                'video_lowq':low_quality,
                 'file_length':file_length,
-                'xurl':xurl,
-                'v_id':v_id,
-                'low_quality':low_quality,
-                'file_length_callb':self._callb_check_file_len,
-                'say_callb':self._say,
-                'opts':self._opts,
-            }
-
+                'callb_filelen':self._callb_check_file_len,
+                'callb_say':self._say}
             self._say("\n")
             v_info = PageParser().parse(url_data,
                 self._found_urls, self._get_proxy())
-
             if self._opts.enable_cache and xurl_expired:
                 self._cache.update_expired_xurl(url, v_info['xurl'])
-
             if self._opts.enable_extract and not self._opts.emit_csv:
-                fn = os.path.basename(v_info['output_file'])
+                fn = self._os.path.basename(v_info['output_file'])
                 text = fn[:70].ljust(70)
-                text += Progress(self._opts, v_info)._byteshuman( \
+                text += Progress(v_info)._byteshuman( \
                     v_info['length_bytes']).rjust(9)
                 self._say(text)
             self._found_urls.append(v_info)
-        except URLGrabError, err:
+        except self._URLGrabError, err:
             self._say('%s [%s]' % (err.strerror,url), is_error=1)
             self._error_urls.append((url, err.strerror))
         except CliveError, err:
@@ -387,7 +354,7 @@ class Nomad:
             self._error_urls.append((url, str(err)))
 
     def _callb_check_file_len(self, xurl):
-        g = URLGrabber(
+        g = self._URLGrabber(
             user_agent = self._opts.http_agent,
             throttle = self._opts.http_throttle,
             proxies = self._get_proxy())
@@ -399,7 +366,7 @@ class Nomad:
             hdrs = o.hdr.items()
             l = o.hdr.get('content-length')
             if l: bytes = long(l)
-        except URLGrabError, e:
+        except self._URLGrabError, e:
             if 'youtube.' in xurl.lower():
                 if e.errno == 14 and e.code == 415:
                     raise CliveNoMediaError
@@ -419,11 +386,9 @@ class Nomad:
         return (mbytes, bytes)
 
     def _show_queue(self, raw_urls=None):
-        opts = Options()
         total_bytes = 0
         total_count = 0
         total_skipped = 0
-
         for (index, v_info) in enumerate(self._found_urls):
             if not v_info['skip_extraction']:
                 total_count += 1
@@ -432,12 +397,10 @@ class Nomad:
                     self._cache.write(self._found_urls[index])
             else:
                 total_skipped += 1
-
-        self._say('=> %d (%s), failed: %d, skipped: %d.' % (
+        self._say('=> %d (%s), failed: %d, skipped: %d' % (
             total_count,
-            Progress(self._opts, None)._byteshuman(total_bytes), 
+            Progress(None)._byteshuman(total_bytes), 
             len(self._error_urls), total_skipped))
-
         if self._opts.enable_recall:
             if total_count > 0 and raw_urls:
                 Recall().write(raw_urls)
@@ -446,10 +409,10 @@ class Nomad:
                     Recall().write(raw_urls)
 
     def _dl_video(self, v_info, omit_say=0):
-        g = URLGrabber(
+        g = self._URLGrabber(
             user_agent = self._opts.http_agent,
             throttle = self._opts.http_throttle,
-            progress_obj = Progress(self._opts, v_info),
+            progress_obj = Progress(v_info),
             proxies = self._get_proxy(),
             reget = v_info['urlg_reget'])
             #opener = self._cookie_opener)
