@@ -4,74 +4,73 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## The classes for parsing command line options
 
-import os
+# Do not cache these imports: they are not used in any other files
 import platform
-import random
 import locale
 from optparse import OptionParser, OptionGroup
 
-import clive as _clive
-from clive.config import ConfigParser
-try:
-    import clive.urlgrabber as urlg
-    from clive.feedparser import feedparser as fparser
-except ImportError, err:
-    raise SystemExit('error: %s' % err)
-
-try:
-    import sqlite3
-    _sqlite3_found = 1
-except ImportError:
-    _sqlite3_found = 0
-
 __all__ = ['Options']
 
+import clive as _clive
+from clive.config import ConfigParser
+from clive.modules import Modules
+from clive.singleton import Singleton
 
 ## The class that parses command line options
-class Options:
+class Options(Singleton):
 
     ## Constructor
     def __init__(self):
         (lang, self._encoding) = locale.getdefaultlocale()
-        if not self._encoding:
-            self._encoding = 'iso-8859-1'
-        self._ver = 'clive %s %s  [%s]\n' \
-            'python/%s; urlgrabber/%s; feedparser/%s' % (
-            _clive.__version__, _clive.__date__.replace('-',''),
-            platform.system(), platform.python_version(),
-            urlg.__version__, fparser.__version__)
-        if len(_clive.__newt_version__) > 0:
-            self._ver += '; newt/%s' % _clive.__newt_version__
-        self._ver += '\n%s/%s; locale=%s' % ( \
-            platform.system(), platform.release().split('-',1)[0],
-            self._encoding)
+        if not self._encoding: self._encoding = 'iso-8859-1'
+        m = Modules()
+        self._ver = [
+            'clive %s%s %s  [%s]\n' % (
+                _clive.__version__, _clive.__version_alphanum__,
+                _clive.__date__.replace('-',''),
+                platform.system()
+            ),
+            '%s/%s; python/%s; locale/%s; build/%s\n' % (
+                platform.system(), platform.release().split('-',1)[0],
+                platform.python_version(), self._encoding,
+                _clive.__build_date__.replace('-','')
+            ),
+            'urlgrabber/%s; pysqlite/%s; feedparser/%s; pycrypto/%s; ' \
+            'newt/%s\n' % (
+                m.check('urlgrabber')['version'],
+                m.check('pysqlite')['version'],
+                m.check('feedparser')['version'],
+                m.check('pycrypto')['version'],
+                m.check('newt')['version']
+            ),
+        ]
         self._warnings = []
 
     ## Parses options
     def parse(self):
         self._parse_opts()
-        return (self.opts, self.args)
 
     ## Print some options to stdout
     def show(self, say):
-        say(self._ver.split('\n',1)[0])
+        say(self._ver[0])
         for w in self._warnings: say(w)
 
     def _parse_opts(self):
@@ -81,7 +80,7 @@ class Options:
         p = OptionParser(
             usage='%prog [OPTION].. [URL]..',
             description=_clive.__doc__,
-            version=self._ver+'\n'+_clive.__notice__)
+            version=''.join(self._ver)+'\n'+_clive.__notice__)
 
         gout = OptionGroup(p, 'output', None)
         gconn = OptionGroup(p, 'connection', None)
@@ -239,6 +238,26 @@ class Options:
                 'desc':'clear last batch data from ~/.clive/recall',
                 'default':False,
             },
+            'enable_confirm':{
+                'group':p,
+                'l_opt':'--no-confirm',
+                's_opt':'-Q',
+                'action':'store_false',
+                'choices':None,
+                'metavar':None,
+                'desc':'do not prompt for approval before each action',
+                'default':True,
+            },
+            'list_hosts':{
+                'group':p,
+                'l_opt':'--hosts',
+                's_opt':'-H',
+                'action':'store_true',
+                'choices':None,
+                'metavar':None,
+                'desc':'list supported hosts',
+                'default':False,
+            },
             # Output group
             'output_exists':{
                 'group':gout,
@@ -301,6 +320,16 @@ class Options:
                 'desc':'proxy for http connections',
                 'default':confp.get_default('http_proxy'),
             },
+            'enable_proxy':{
+                'group':gconn,
+                'l_opt':'--no-proxy',
+                's_opt':'-N',
+                'action':'store_false',
+                'choices':None,
+                'metavar':None,
+                'desc':'disable use of http proxy altogether',
+                'default':True,
+            },
             'http_agent':{
                 'group':gconn,
                 'l_opt':'--agent',
@@ -321,45 +350,45 @@ class Options:
                 'desc':'throttle limit for transfers (kb/s)',
                 'default':confp.get_default('http_throttle'),
             },
-            'youtube_username':{
+            'youtube_login':{
                 'group':gconn,
-                'l_opt':'--youtube-user',
+                'l_opt':'--youtube',
                 's_opt':None,
                 'action':'store',
                 'choices':None,
-                'metavar':'U',
-                'desc':'youtube log-in username',
-                'default':confp.get_default('youtube_username'),
+                'metavar':'LOGINFO',
+                'desc':'youtube login in format "username:password"',
+                'default':confp.get_default('youtube_login'),
             },
-            'youtube_password':{
+            'dmotion_login':{
                 'group':gconn,
-                'l_opt':'--youtube-pass',
+                'l_opt':'--dmotion',
                 's_opt':None,
                 'action':'store',
                 'choices':None,
+                'metavar':'LOGINFO',
+                'desc':'dailymotion login in format "username:password"',
+                'default':confp.get_default('dmotion_login'),
+            },
+            'enable_login':{
+                'group':gconn,
+                'l_opt':'--no-login',
+                's_opt':'-G',
+                'action':'store_false',
+                'choices':None,
+                'metavar':None,
+                'desc':'disable logging in altogether',
+                'default':True,
+            },
+            'passwd_passphrase':{
+                'group':gconn,
+                'l_opt':'--passwd',
+                's_opt':'-P',
+                'action':'store',
+                'choices':None,
                 'metavar':'P',
-                'desc':'youtube log-in password',
-                'default':confp.get_default('youtube_password'),
-            },
-            'dmotion_username':{
-                'group':gconn,
-                'l_opt':'--dmotion-user',
-                's_opt':None,
-                'action':'store',
-                'choices':None,
-                'metavar':'U',
-                'desc':'dailymotion log-in username',
-                'default':confp.get_default('dmotion_username'),
-            },
-            'dmotion_password':{
-                'group':gconn,
-                'l_opt':'--dmotion-pass',
-                's_opt':None,
-                'action':'store',
-                'choices':None,
-                'metavar':'P',
-                'desc':'dailymotion log-in password',
-                'default':confp.get_default('dmotion_password'),
+                'desc':'use passphrase for ~/.clive/passwd w/o prompting',
+                'default':None,
             },
             # Playback options
             'player':{
@@ -425,7 +454,6 @@ class Options:
                 'default':confp.get_default('enable_xclip_paste'),
             },
         }
-
         for (opt, data) in sorted(lookup.items()):
             o = lookup[opt]
             o['group'].add_option(
@@ -439,129 +467,155 @@ class Options:
                 help=o['desc']
             )
 
-        (self.opts, self.args) = p.parse_args()
+        (self._opts, self._args) = p.parse_args()
+
+        os = Modules().getinst('os')
 
         # Modify: proxy -- use http_proxy
         proxy = os.environ.get('http_proxy')
-        if proxy and not self.opts.http_proxy:
-            self.opts.http_proxy = proxy
+        if proxy and not self._opts.http_proxy:
+            self._opts.http_proxy = proxy
+        if self._opts.http_proxy and not self._opts.enable_proxy:
+            self._opts.http_proxy = None # Disable proxy altogether
+            try:
+                del os.environ['http_proxy'] # Otherwise urlgrabber will use it
+            except KeyError:
+                pass
 
         # Modify: savedir
         w = 'warn: invalid savedir, using workdir'
-        if not self.opts.output_savedir or len(self.opts.output_savedir) == 0:
-            self.opts.output_savedir = os.getcwd()
+        if not self._opts.output_savedir or len(self._opts.output_savedir) == 0:
+            self._opts.output_savedir = os.getcwd()
             self._warnings.append(w)
 
-        if not os.path.exists(self.opts.output_savedir):
-            self.opts.output_savedir = os.getcwd()
+        if not os.path.exists(self._opts.output_savedir):
+            self._opts.output_savedir = os.getcwd()
             self._warnings.append(w)
 
         # Modify: user-agent
-        if not self.opts.http_agent:
-            self.opts.http_agent = self._random_agent()
-        if len(self.opts.http_agent) == 0:
-            self.opts.http_agent = self._random_agent()
+        if not self._opts.http_agent:
+            self._opts.http_agent = self._random_agent()
+        if len(self._opts.http_agent) == 0:
+            self._opts.http_agent = self._random_agent()
             self._warnings.append('warn: invalid user-agent, using random')
 
         # Modify: throttle (KB/s -> Bytes/s for urlgrabber)
-        if not self.opts.http_throttle: self.opts.http_throttle = 0
+        if not self._opts.http_throttle: self._opts.http_throttle = 0
         try:
-            self.opts.http_throttle = int(self.opts.http_throttle) * 1024
+            self._opts.http_throttle = int(self._opts.http_throttle) * 1024
         except ValueError:
-            self.opts.http_throttle = 0
+            self._opts.http_throttle = 0
             self._warnings.append('warn: invalid throttle, using default')
 
         # Modify: play, player, ffmpeg and reencode
-        if not self.opts.player or len(self.opts.player) == 0:
-            if self.opts.play_format:
+        if not self._opts.player or len(self._opts.player) == 0:
+            if self._opts.play_format:
                 self._warnings.append('warn: --player unused, --play ignored')
-                self.opts.play_format = None
+                self._opts.play_format = None
 
-        if not self.opts.ffmpeg or len(self.opts.ffmpeg) == 0:
-            if self.opts.reencode_format:
+        if not self._opts.ffmpeg or len(self._opts.ffmpeg) == 0:
+            if self._opts.reencode_format:
                 self._warnings.append('warn: --ffmpeg unused, ' +
                     '--reencode ignored')
-                self.opts.reencode_format = None                    
+                self._opts.reencode_format = None                    
 
-        if self.opts.play_format and self.opts.play_format != 'src':
-            if self.opts.ffmpeg and len(self.opts.ffmpeg) > 0:
-                self.opts.reencode_format = self.opts.play_format
+        if self._opts.play_format and self._opts.play_format != 'src':
+            if self._opts.ffmpeg and len(self._opts.ffmpeg) > 0:
+                self._opts.reencode_format = self._opts.play_format
             else:
-                self.opts.play_format = 'src'
-                self.opts.reencode_format = None
+                self._opts.play_format = 'src'
+                self._opts.reencode_format = None
                 self._warnings.append('warn: --ffmpeg unused, using --play=src')
 
-        if self.opts.player:
-            if '%i' not in self.opts.player:
-                if self.opts.play_format:
+        if self._opts.player:
+            if '%i' not in self._opts.player:
+                if self._opts.play_format:
                     self._warnings.append('warn: --player missing "%i", ' +
                         'ignoring --play')
-                    if self.opts.play_format != 'src':
-                        self.opts.reencode_format = None
+                    if self._opts.play_format != 'src':
+                        self._opts.reencode_format = None
                         self._warnings.append('warn: --reencode ignored ' +
                             'because of the above')
-                    self.opts.play_format = None
+                    self._opts.play_format = None
 
-        if self.opts.ffmpeg:
+        if self._opts.ffmpeg:
             for s in ['%i','%o']:
-                if s not in self.opts.ffmpeg:
-                    if self.opts.reencode_format:
+                if s not in self._opts.ffmpeg:
+                    if self._opts.reencode_format:
                         self._warnings.append('warn: --ffmpeg missing "%s", ' \
                             '--reencode ignored' % s)
-                        self.opts.reencode_format = None
-                        if self.opts.play_format and \
-                            self.opts.play_format != 'src':
+                        self._opts.reencode_format = None
+                        if self._opts.play_format and \
+                            self._opts.play_format != 'src':
                             self._warnings.append('warn: using --play=src ' +
                                 'instead')
-                            self.opts.play_format = 'src'
+                            self._opts.play_format = 'src'
 
         # Modify: xclip
-        if self.opts.enable_xclip_paste:
-            if not self.opts.xclip:
-                self.opts.enable_xclip_paste = False
+        if self._opts.enable_xclip_paste:
+            if not self._opts.xclip:
+                self._opts.enable_xclip_paste = False
                 self._warnings.append('warn: --xclip unused, ignoring --paste')
             else:                
-                if '-o' not in self.opts.xclip:
-                    self.opts.xclip += ' -o'
+                if '-o' not in self._opts.xclip:
+                    self._opts.xclip += ' -o'
                     self._warnings.append('warn: --xclip missing "-o", ' +
                         'appending it')
 
+        mods = Modules()
+
         # Modify: cache
-        if not _sqlite3_found:
-            self.opts.enable_cache = False
+        if not mods.check('pysqlite')['installed']:
+            self._opts.enable_cache = False
 
-        # Modify: youtube-*
-        if self.opts.youtube_username:
-            if not self.opts.youtube_password:
-                self.opts.youtube_username= None
-                self._warnings.append('warn: --youtube-pass unused, ' +
-                    'ignoring --youtube-user')
-        if self.opts.youtube_password:
-            if not self.opts.youtube_username:
-                self.opts.youtube_password = None
-                self._warnings.append('warn: --youtube-user unused, ' +
-                    'ignoring --youtube-pass')
+        # Modify: cache browser
+        if self._opts.enable_cache_browse:
+            m = mods.check('pysqlite')
+            if not m['installed']:
+                raise SystemExit('error: %s' % m['error'])
 
-        # Modify: dmotion-*
-        if self.opts.dmotion_username:
-            if not self.opts.dmotion_password:
-                self.opts.dmotion_username = None
-                self._warnings.append('warn: --dmotion-pass unused, ' +
-                    'ignoring --dmotion-user')
-        if self.opts.dmotion_password:
-            if not self.opts.dmotion_username:
-                self.opts.dmotion_password = None
-                self._warnings.append('warn: --dmotion-user unused, ' +
-                    'ignoring --dmotion-pass')
+        # Modify: scan, rss, cache (newt)
+        if self._opts.enable_scan \
+        or self._opts.enable_rss \
+        or self._opts.enable_cache_browse:
+            m = mods.check('newt')
+            if not m['installed']:
+                raise SystemExit('error: %s' % m['error'])
+
+        # Modify: rss
+        if self._opts.enable_rss:
+            m = mods.check('feedparser')
+            if not m['installed']:
+                raise SystemExit('error: %s' % m['error'])
+
+        # Modify: youtube login
+        if self._opts.youtube_login:
+            if not self._opts.enable_login: # Turn it off
+                self._opts.youtube_login = None
+            else: # Do a basic string format check
+                if not ':' in self._opts.youtube_login:
+                    self._opts.youtube_login = None
+                    self._warnings.append('warn: ignored --youtube, ' +
+                        'login info needs to be in "username:password" format')
+
+        # Modify: dmotion login
+        if self._opts.dmotion_login:
+            if not self._opts.enable_login: # Turn it off
+                self._opts.dmotion_login = None
+            else: # Do a basic string format check
+                if not ':' in self._opts.dmotion_login:
+                    self._opts.dmotion_login = None
+                    self._warnings.append('warn: ignored --dmotion, ' +
+                        'login info needs to be in "username:password" format')
 
         # Check for default locale settings which are used for string
         # conversions.
-        setattr(self.opts, 'encoding', self._encoding) # Add artificially
+        setattr(self._opts, 'encoding', self._encoding) # Add artificially
 
         # Modify: Append "artificially" feed_url attribute to the options
         # The "feed_url" a sequence of RSS/Atom URLs from the command line
         # or the config file.
-        setattr(self.opts, 'feed_url', confp.get_default('feed_url'))
+        setattr(self._opts, 'feed_url', confp.get_default('feed_url'))
 
     def _random_agent(self, pairs=4, pair_len=2):
         lst = ['clive', 'youtube', 'google', 'motion',
@@ -574,6 +628,7 @@ class Options:
                 i=l; l+=pair_len;
                 if i >= len(w): break
         t=''; l=-1;
+        random = Modules().getinst('random')
         for i in range(pairs):
             while 1:
                 j = random.randint(0, len(a)-1)
@@ -583,8 +638,3 @@ class Options:
         mi = random.randint(0,9)
         rv = random.randint(0,9)
         return '%s/%d.%d.%d' % (t,ma,mi,rv)
-
-if __name__ == '__main__':
-    o = Options()
-    opts, args = o.parse()
-    print opts
