@@ -4,51 +4,38 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## Classes for parsing video page HTML
 
-import os
-import sys
+# Do not cache these imports: they are not used in any other files
 import urllib
-import time
-import md5
 import re
-import string
-import gzip
-
 from sgmllib import SGMLParser
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+__all__ = ['PageParser', 'Video']
 
 from clive.path import ConfigDir
 from clive.unicode import tostr, touni
+from clive.modules import Modules
+from clive.opts import Options
 from clive.error import CliveError, CliveNoMediaError
-try:
-    from clive.urlgrabber.grabber import URLGrabber, URLGrabError
-except ImportError, err:
-    raise SystemExit('error: %s' % err)
 
-__all__ = ['PageParser']
-
-
-## Parses video page title tag
+## Parses video page title
 class TitleParser(SGMLParser):
     def reset(self):
         SGMLParser.reset(self)
@@ -70,6 +57,9 @@ class PageParser:
 
     ## Constructor
     def __init__(self):
+        m = Modules()
+        (self._URLGrabber, self._URLGrabError) = m.getinst('urlgrabber')
+        self._os = m.getinst('os')
         self._supported_hosts = [
             ('youtube.com', 'ytube', self._parse_ytube),
             ('video.google.', 'vgoogle', self._parse_vgoogle),
@@ -77,104 +67,91 @@ class PageParser:
             ('guba.com', 'guba', self._parse_guba),
             ('metacafe.', 'metac', self._parse_metacafe),
             ('sevenload.com', 'sevenl', self._parse_sevenload),
-            ('myvideo.', 'myvideo', self._parse_myvideo),
-            #('break.com', 'break', self._parse_break),
+            #('myvideo.', 'myvideo', self._parse_myvideo),
+            ('break.com', 'break', self._parse_break),
         ]
 
     ## Parses a video page data (HTML)
     def parse(self, url_data, batch, proxy):
-        self._say = url_data['say_callb']
-        self._opts = url_data['opts']
+        self._say = url_data['callb_say']
+        self._opts = Options()._opts
         self._proxy = proxy
-
-        if len(url_data['title']) == 0:
+        if len(url_data['page_title']) == 0:
             p = TitleParser()
-            p.feed(url_data['data'])
+            p.feed(url_data['page_data'])
             p.close()
             title = touni(p.title)
         else:
-            title = url_data['title']
-
+            title = url_data['page_title']
         xurl = '' # video eXtraction URL
-        low_quality = url_data['low_quality']
+        low_quality = url_data['video_lowq'] # TODO: CLEANUP
         for (site, v_host, func) in self._supported_hosts:
-            if site in url_data['url'].lower():
-                if len(url_data['xurl']) > 0:
-                    xurl = url_data['xurl'] # From cache
-                    v_id = url_data['v_id']
+            if site in url_data['page_url'].lower():
+                if len(url_data['page_xurl']) > 0:
+                    xurl = url_data['page_xurl'] # From cache
+                    v_id = url_data['video_id']
                 else:
                     (xurl, v_id, low_quality) = \
-                        func(url_data['url'], url_data['data'], low_quality)
+                        func(url_data['page_url'], url_data['page_data'], low_quality)
                 break
-
         if len(xurl) == 0:
             raise CliveError('error: extraction url not found')
-
         if url_data['file_length'] == -1:
             try:
-                length = url_data['file_length_callb'](xurl)
+                length = url_data['callb_filelen'](xurl)
             except CliveNoMediaError: # mp4 not avail., fallback to flv
                 self._say('warn: no-media: switched to low-quality')
                 xurl = xurl.strip('&fmt=18')
-                length = url_data['file_length_callb'](xurl)
+                length = url_data['callb_filelen'](xurl)
                 low_quality = 1 # !
         else:
             l = url_data['file_length']
             length = ('%.2fMB' % (float(l) / (1024*1024)), l)
-
         v_info = {
             'page_title':title,
-            'url':url_data['url'],
+            'url':url_data['page_url'],
             'xurl':xurl,
             'v_id':v_id,
             'v_host':v_host,
             'length':length[0],
             'length_bytes':length[1],
-            'low_quality':low_quality,
-        }
+            'low_quality':low_quality}
         self._get_filename(v_info, batch)
         return v_info
 
     def _parse_ytube(self, url, data, low_quality):
-        try:
-            vid = url.split('watch?v=',1)[1].split('&',1)[0]
-        except IndexError:
-            vid = self._random_vid()
-
-        ldata = data.lower()
-
-        a = [('verify you are 18','age verification: use --youtube-user'),
-            ('no longer available','not available'),
-            #('has been removed','video removed'),
-            ('video is unavailable','video unavailable'),
-            ('malformed video id','url contains malformed video id'),
-            ('not available in your country','country restriction/censorship'),
-            ('private video','private video')]
-
+        # More headache than use. Ignorant of non-English errors.
+        """
+        a = [
+        ('verify you are 18','age verification: use --youtube'),
+        ('no longer available','not available'),
+        #('has been removed','video removed'),
+        ('video is unavailable','video unavailable'),
+        ('malformed video id','url contains malformed video id'),
+        ('not available in your country','country restriction/censorship'),
+        #('private video','private video')]
+        ]
         for (lookup, errmsg) in a:
-            if lookup in ldata:
+            if lookup in data.lower():
                 raise CliveError('error: ' + errmsg)
-        try:
+        """                
+        try: # &video_id param
             video_id = \
                 self._parse_from_to(data, 'video_id=', '&', skip_from=1)
             video_id = video_id.replace("'", "")
             if len(video_id) == 0: raise CliveError()
         except:
             raise CliveError('error: extraction url (&video_id) not found')
-		
-        try:
+        try: # &t param
             t = self._parse_from_to(data, '&t=', '&', skip_from=1)
             t = t.replace("'", "")
             if len(t) == 0: raise CliveError()
         except:
             raise CliveError('error: extraction url (&t) not found')
-
         url = 'http://www.youtube.com/get_video?video_id=%s&t=%s' % (
             video_id, t)
-
-        if not self._opts.enable_low_quality:
-            url += '&fmt=18'
-        return (url, vid, low_quality)
+        if not self._opts.enable_low_quality: url += '&fmt=18'
+        return (url, video_id, low_quality)
 
     def _parse_vgoogle(self, url, data, low_quality):
         try:
@@ -182,11 +159,10 @@ class PageParser:
         except IndexError:
             vid = self._random_vid()
         data = data.decode('unicode-escape','ignore')
-        data = tostr(data, self._opts)
+        data = tostr(data)
         # flv
         url = self._parse_from_to(data,
-            #data.decode('unicode-escape','ignore').encode('latin-1','ignore'),
-            'googleplayer.swf?videoUrl=', '"', skip_from=1)
+            'googleplayer.swf?videoUrl\x3d', '\x26', skip_from=1)
         url = urllib.unquote(url)
         try:
             url = url.split('&thumb',1)[0]
@@ -212,8 +188,7 @@ class PageParser:
         if len(batch) == 0:
             ldata = data.lower()
             a = [('content deleted','video removed'),
-                ('users+have+flagged','age verification: use --dmotion-user')]
-                #('explicit content','no-support; req. age verification')]
+                ('users+have+flagged','age verification: use --dmotion')]
             for (lookup, errmsg) in a:
                 if lookup in ldata:
                     raise CliveError('error: ' + errmsg)
@@ -287,7 +262,7 @@ class PageParser:
             vid = self._random_vid()
         url = self._parse_from_to(data, '.swf?', '&', skip_from=1)
         return (url, vid, low_quality)
-    """
+
     def _parse_break(self, url, data, low_quality):
         vid = self._parse_from_to(data,
             "GlobalContentID='", "'", skip_from=1)
@@ -296,25 +271,28 @@ class PageParser:
         name = self._parse_from_to(data,
             "FileName='", "'", skip_from=1)            
         url = 'http://media1.break.com/dnet/media/%s/%s' % (path,name)
-        if low_quality:
-            url += '.flv'
-        else:
-            url += '.wmv'
-        return (url, vid, low_quality)            
-    """
+        if low_quality: url += '.flv'
+        else: url += '.wmv'
+        return (url, vid, low_quality)
+
     def _fetch_page(self, url):
-        g = URLGrabber(user_agent = self._opts.http_agent,
+        g = self._URLGrabber(user_agent = self._opts.http_agent,
             http_headers = (('accept-encoding', 'gzip'),),
             throttle = self._opts.http_throttle,
             proxies = self._proxy)
         o = g.urlopen(url)
         data = o.read()
         if o.hdr.get('content-encoding') == 'gzip':
+            m = Modules()
+            gzip = m.getinst('gzip')
+            StringIO = m.getinst('StringIO')
             data=gzip.GzipFile(fileobj=StringIO(data)).read()
         o.close()
         return data
 
     def _random_vid(self):
+        md5 = Modules().getinst('md5')
+        time = Modules().getinst('time')
         return md5.new(str(time.time())).hexdigest()[:8]
 
     def _parse_from_to(self, data, _from, to, skip_from=0):
@@ -332,14 +310,12 @@ class PageParser:
         skip = 0
         ext = 'flv'
         offset = 0
-
         if v_info['v_host'] in ['ytube','dmotion','vgoogle']:
             if not v_info['low_quality']:
                 ext = 'mp4'
                 if 'dailymotion.' in v_info['url'].lower():
                     end = v_info['xurl'].find('?') # copy ext from page html
                     ext = v_info['xurl'][end-3:end]
-
         title = v_info['page_title'].replace('YouTube -', '')
         title = title.replace('GUBA -', '')
         title = title.replace(' Video - Metacafe', '')
@@ -353,18 +329,15 @@ class PageParser:
             title = title.replace('" | sevenload','')
         if v_info['v_host'] == 'myvideo':
             title = v_info['page_title'].split('-',1)[0].rstrip()
-        """            
         if v_info['v_host'] == 'break':
             title = title.rstrip('Video')
             if not v_info['low_quality']:
                 ext = 'wmv'
-        """
         # Apply these title exceptions to all inspite of custom filters etc.
         title = title.replace('/', '_') # Will definitely cause issues otherwise
-
         # Use user-defined custom method from ~/.clive/custom.py
         if self._opts.output_filter == 'custom':
-            
+            sys = Modules().getinst('sys')
             sys.path.append(ConfigDir().dir())
             try:
                 from custom import custom_output_filter
@@ -377,77 +350,64 @@ class PageParser:
             self._opts.output_filter.lower() != 'no':
                 # Treat string as a regexp for re.sub
                 title = re.sub('[^%s]' % self._opts.output_filter, '', title)
-
         title = title.lstrip().rstrip()
         if len(title) == 0:
             title = self._random_string(insert_dash=0)
-
         filename = self._opts.output_format.replace('%','$')
         d = {'t':title,'i':v_info['v_id'],'h':v_info['v_host'],'e':ext}
-        filename = string.Template(filename).substitute(d)
-        filename = tostr(filename, self._opts)
-
+        string = Modules().getinst('string')
+        filename = tostr(string.Template(filename).substitute(d))
         if self._opts.output_file:
             filename = self._opts.output_file
-
         if self._opts.output_savedir:
-            filename = os.path.join(self._opts.output_savedir, filename)
-
-        if os.path.exists(filename) and not self._opts.emit_csv:
-            if os.path.getsize(filename) < v_info['length_bytes']:
-                a = ['dmotion', 'guba', 'metac', 'ytube',
-                    'vgoogle', 'sevenl', 'myvideo',
-                    #'break'
-                    ]
-                if ext == 'flv':
-                    # Continue partial download fails for flv files
-                    # (ytube, vgoogle). mp4 transfers are ok.
-                    if v_info['v_host'] == 'ytube':
-                        a.remove('ytube')
-                    if v_info['v_host'] == 'vgoogle':
-                        a.remove('vgoogle')
-                if v_info['v_host'] in a:
-                    urlg_reget = 'simple' # Continue partial download
-                    offset = os.path.getsize(filename)
-                else:
+            filename = self._os.path.join(self._opts.output_savedir, filename)
+        if self._os.path.exists(filename) and not self._opts.emit_csv:
+            # If local_file < remote_file
+            if self._os.path.getsize(filename) < v_info['length_bytes']:
+                allows_continue = 1 # Most hosts allow by default
+                if ext == 'flv': # Youtube/FLV, Vgoogle/FLV exceptions
+                    if v_info['v_host'] in ['ytube','vgoogle']:
+                        allows_continue = 0 # Disable
+                if allows_continue: # Continue partial download
+                    urlg_reget = 'simple'
+                    offset = self._os.path.getsize(filename)
+                else: # Overwrite or rename
                     if self._opts.output_exists != 'overwrite':
                         self._say('warn: no-support; cannot continue ' +
                             'partially downloaded file')
                         # Continue getting a partially-downloaded file
                         filename = self._rename_file(filename)
-            else:
+            else: # If local_file >= remote_file
                 if self._opts.output_exists == 'overwrite':
-                    os.remove(filename)
+                    self._os.remove(filename)
                 else:    
-                    if os.path.getsize(filename) == v_info['length_bytes']:
+                    if self._os.path.getsize(filename) == \
+                    v_info['length_bytes']:
                         skip = 1
                         self._say('warn: skipped: %s (same length) ' \
-                            'exists already' % os.path.basename(filename))
+                            'exists already' % self._os.path.basename(filename))
                     else:                                
                         filename = self._rename_file(filename)
-        
         # Make sure there are not any duplicates in the URL batch
         for vi in batch:
             if vi['output_file'] == filename:
                 filename = self._rename_file(filename) # Found
                 break
-
         v_info['output_file'] = filename
         v_info['urlg_reget'] = urlg_reget
         v_info['skip_extraction'] = skip
         v_info['offset'] = offset
 	
     def _rename_file(self, filename):
-        (root, ext) = os.path.splitext(filename)
+        (root, ext) = self._os.path.splitext(filename)
         return root + self._random_string() + ext
 
     def _random_string(self, insert_dash=1):
         s = ''
         if insert_dash: s += '-'
+        time = Modules().getinst('time')
+        md5 = Modules().getinst('md5')
         s += (md5.new(str(time.time())).hexdigest()[:8])
         return s
-        """
-        s += time.strftime('-%Y-%m-%dT%H:%M:%S').replace(':', '_')
-        """
 
 
