@@ -4,29 +4,27 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## The classes that involve locating or constructing file paths
 
-import os
-import subprocess
-import errno
-
 __all__ = ['ConfigDir', 'Autodetect']
 
+from clive.modules import Modules
 
 ## The class that wraps absolute paths to config etc. files
 class ConfigDir:
@@ -35,21 +33,23 @@ class ConfigDir:
     _cachefile = 'cache'
     _recallfile = 'recall'
     _customfile = 'custom.py'
+    _passwdfile = 'passwd'
 
     ## Constructor
     def __init__(self):
+        os = Modules().getinst('os')
         self._rcdir = os.path.join(os.path.expanduser('~'), self._rcdir)
-
+        errno = Modules().getinst('errno')
         try:
             os.makedirs(self._rcdir)
         except OSError, e:
             if e.errno == errno.EEXIST: pass
             else: raise
-
         self._rcfile = os.path.join(self._rcdir, self._rcfile)
         self._cachefile = os.path.join(self._rcdir, self._cachefile)
         self._recallfile = os.path.join(self._rcdir, self._recallfile)
         self._customfile = os.path.join(self._rcdir, self._customfile)
+        self._passwdfile = os.path.join(self._rcdir, self._passwdfile)
 
     ## Return path to ~/.clive directory
     def dir(self):
@@ -71,16 +71,24 @@ class ConfigDir:
     def customfile(self):
         return self._customfile
 
+    ## Return path to passwd file
+    def passwdfile(self):
+        return self._passwdfile
+
 
 ## The class used to locate programs
 class Autodetect:
     ## Return path to a program
-    def locate(self, cmd, append="", say=None):
-        pipe = subprocess.Popen('which %s 2>/dev/null' % cmd,
-            shell=1, stdout=subprocess.PIPE).stdout
-        data = pipe.read().rstrip('\n')
+    def locate(self, cmds, append="", say=None):
+        if not isinstance(cmds, list): cmds = [cmds] # -> list
         path = None
-        if len(data) > 0:
-            path = '%s %s' % (data, append)
-            if say: say('found: %s' % data)
+        subprocess = Modules().getinst('subprocess')
+        for cmd in cmds:
+            pipe = subprocess.Popen('which %s 2>/dev/null' % cmd,
+                shell=1, stdout=subprocess.PIPE).stdout
+            data = pipe.read().rstrip('\n')
+            if len(data) > 0:
+                path = '%s %s' % (data, append)
+                if say: say('found: %s' % data)
+                break # use first found item from the cmd list
         return path
