@@ -4,53 +4,50 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 ## The classes for parsing rss/atom feeds
 
-import urllib2
-
-try:
-    from clive.feedparser import feedparser as fparser
-    from clive.newt import snack as newt
-except ImportError, err:
-    raise SystemExit('error: %s' % err)
+__all__ = ['FeedParser']
 
 from clive.unicode import tostr
-
-__all__ = ['FeedParser']
+from clive.modules import Modules
+from clive.opts import Options
 
 ## The feed parsing class
 class FeedParser:
 
     ## Constructor
-    def __init__(self, opts, say, proxy):
-        self._opts = opts
+    def __init__(self, say, proxy):
+        self._opts = Options()._opts
         self._say = say
         self._proxy = proxy
+        self._fparser = Modules().getinst('feedparser')
 
     ## Reap feeds and display selection lists
     def parse(self, raw_urls):
-        fparser.USER_AGENT = self._opts.http_agent
+        self._fparser.USER_AGENT = self._opts.http_agent
+        urllib2 = Modules().getinst('urllib2')
         h = urllib2.ProxyHandler(self._proxy)
         # Reap URLs
         feeds = []
         for url in raw_urls:
             self._say('feed: %s' % url)
-            d = fparser.parse(url, handlers=[h])
+            d = self._fparser.parse(url, handlers=[h])
             if not hasattr(d.feed, 'title'):
                 self._say('error: invalid rss/atom feed')
             elif not hasattr(d, 'entries'):
@@ -60,15 +57,22 @@ class FeedParser:
         # Display feeds and return user selection (video URLs)
         if len(feeds) == 0:
             raise SystemExit('error: no data: check feed url(s)')
-        u = DisplayFeeds(feeds, self._opts)
-        return u.get_selection()
-
+        if self._opts.enable_confirm:            
+            u = DisplayFeeds(feeds, self._opts)
+            return u.get_selection()
+        else: # Skip selection and use the whole list
+            sel = []
+            for d in feeds:
+                for e in d.entries:
+                    sel.append(e.link)
+            return sel                    
 
 ## The class that displays the feeds with found video URLs
 class DisplayFeeds:
 
     ## Constructor
     def __init__(self, feeds, opts):
+        self._newt = Modules().getinst('newt')
         self._last = None # Selection before hitting "Resize"
         self._feeds = feeds
         self._opts = opts
@@ -83,7 +87,7 @@ class DisplayFeeds:
     def _main(self):
         sel = []
         try:
-            scr = newt.SnackScreen()
+            scr = self._newt.SnackScreen()
             sel = self._show_feeds(scr)
         finally:
             scr.finish()
@@ -102,33 +106,28 @@ class DisplayFeeds:
     def _show_feeds(self, scr):
         w = scr.width - (scr.width/6)
         ww = w-12
-        ct = newt.CheckboxTree(scr.height/2, scroll=1)
+        ct = self._newt.CheckboxTree(scr.height/2, scroll=1)
         for (i,d) in enumerate(self._feeds):
             t = d.feed.title[:w]
             t += (' (%d)' % len(d.entries))
             try:
-                t = tostr(t, self._opts)
+                t = tostr(t)
             except UnicodeDecodeError, err:
                 continue # Skip item if decoding fails for some reason
             ct.append(t)
             for e in d.entries:
-                """
-                date = e.date_parsed
-                t = ('%02d-%02d-%02d: %s' % \
-                    (date[0],date[1],date[2],e.title))[:ww]
-                """
                 try:
-                    t = tostr(e.title[:ww], self._opts)
+                    t = tostr(e.title[:ww])
                 except UnicodeDecodeError, err:
                     continue # Skip item if decoding fails for some reason
                 sel = False
                 if self._last and e.link in self._last:
                     sel = True
-                ct.addItem(t, (i, newt.snackArgs['append']),
+                ct.addItem(t, (i, self._newt.snackArgs['append']),
                     e.link, selected=sel)
-        g = newt.GridForm(scr, 'RSS/Atom Feeds: Videos found', 1,2)
+        g = self._newt.GridForm(scr, 'RSS/Atom Feeds: Videos found', 1,2)
         g.add(ct, col=0, row=0, padding=(0,0,0,1))
-        b = newt.ButtonBar(scr,
+        b = self._newt.ButtonBar(scr,
             [('Resize',2),('Cancel',0), ('Extract',1)], compact=0)
         g.add(b, col=0, row=1, padding=(0,0,0,0))
         self._ct = ct
