@@ -4,23 +4,26 @@
 # clive, video extraction utility
 # Copyright (C) 2007-2008 Toni Gundogdu
 #
-# clive is free software; you can redistribute it and/or modify
+# This file is part of clive.
+#
+# clive is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# clive is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
+# along with clive.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
 __all__ = ['touni', 'tostr']
 
+from clive.opts import Options
+
 ## Convert object to unicode string (utf-8 by default)
 def touni(s, encoding='utf-8'):
     if isinstance(s,basestring):
@@ -29,8 +32,8 @@ def touni(s, encoding='utf-8'):
     return s
 
 ## Convert unicode string to locale encoding
-def tostr(s, opts):
+def tostr(s):
     if not isinstance(s, unicode):
         s = touni(s)
-    s = s.encode(opts.encoding,'ignore')
+    s = s.encode(Options()._opts.encoding,'ignore')
     return s
